/*-----------------------------------------------------------------------------------------  
  -- Programa:    Agente buscador en web usando stemming
  -- Autor:       Jess Ferrero Leonardo
  -- Fecha:       7-8-2002
  -- Fichero:     Buscador.java
  -- Descripcin: Interfaz grfico del buscador
  -----------------------------------------------------------------------------------------*/

package mwa.agent;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.color.*;
import java.util.*;
import java.io.*;
import java.net.*;

/** 
 * Interfaz grfico del buscador
 */
public class Buscador extends JFrame implements ActionListener {

    private static final int ANCHO=500;
    private static final int ALTO=600;

    // Variables con los componentes
    private JPanel superior;
    private JPanel central;
    private JPanel inferior;

    private JTextField txtKeyword;
    private JTextField txtNotKeyword;
    private JTextField txtUrl;
    private JTextField txtProfundidad;
    
    private JButton buscarCache;
    private JButton buscarRed;
    
    private JTextArea lineas;
    private JScrollPane panel;

    // Instancia del buscador
    Spider spider;

    /** Constructor vacio */
    public Buscador() {
        initComponents();
        spider=new Spider();
    }

    private void initComponents() {
        // Para que al cerrar el formulario se cierre el programa
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        // Inicializa el formulario
        setTitle("Pooble v1.0");
        getContentPane().setLayout(new BorderLayout());
        setSize(ANCHO,ALTO);
        setVisible(false);

        // Inicializa los paneles
        superior=new JPanel();
        superior.setPreferredSize(new Dimension(100,150));
        superior.setBorder(BorderFactory.createCompoundBorder(
                        BorderFactory.createTitledBorder("Parmetros de la bsqueda"),
                        BorderFactory.createEmptyBorder(5,5,5,5)));
        superior.setLayout(new FlowLayout());
        getContentPane().add(superior, java.awt.BorderLayout.NORTH);
        
        central=new JPanel();
        central.setPreferredSize(new Dimension(100,100));
        central.setBorder(BorderFactory.createCompoundBorder(
                        BorderFactory.createTitledBorder("Resultado de la bsqueda"),
                        BorderFactory.createEmptyBorder(5,5,5,5)));
        central.setLayout(new BorderLayout());
        getContentPane().add(central, java.awt.BorderLayout.CENTER);
        
        inferior=new JPanel();
        inferior.setPreferredSize(new Dimension(100,80));
        inferior.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        inferior.setLayout(new FlowLayout());
        getContentPane().add(inferior, java.awt.BorderLayout.SOUTH);
        
        // Etiqueta(s)
        superior.add(new JLabel("URL inicial: "));
        txtUrl= new JTextField("http://www.nasa.gov/",30); superior.add(txtUrl);

        superior.add(new JLabel("Profundidad: "));
        txtProfundidad= new JTextField("1",30); superior.add(txtProfundidad);

        superior.add(new JLabel("Palabras clave: "));
        txtKeyword= new JTextField("solar system",30); superior.add(txtKeyword);

        superior.add(new JLabel("Palabras prohibidas: "));
        txtNotKeyword= new JTextField("",30); superior.add(txtNotKeyword);

        // Botones inferiores
        ImageIcon imgCache = new ImageIcon("cache.gif");
        buscarCache = new JButton("Buscar en cache", imgCache);
        buscarCache.setVerticalTextPosition(AbstractButton.BOTTOM);
        buscarCache.setHorizontalTextPosition(AbstractButton.CENTER);
        buscarCache.setBackground(Color.white);
        buscarCache.setActionCommand("cache");
        buscarCache.addActionListener(this);
        buscarCache.setEnabled(false);
        inferior.add(buscarCache);

        ImageIcon imgRed = new ImageIcon("internet.gif");
        buscarRed = new JButton("Buscar en Internet", imgRed);
        buscarRed.setVerticalTextPosition(AbstractButton.BOTTOM);
        buscarRed.setHorizontalTextPosition(AbstractButton.CENTER);
        buscarRed.setBackground(Color.white);
        buscarRed.setActionCommand("inet");
        buscarRed.addActionListener(this);
        inferior.add(buscarRed);

        // Lneas de progreso
        lineas=new JTextArea(1,30);
        lineas.setEditable(false);
        panel=new JScrollPane(lineas);
        panel.setPreferredSize(new Dimension (100,100));
        central.add(panel);
        
        this.setLocation((int) (1024/2)-(ANCHO/2) , (int) (768/2)-(ALTO/2) );
        //pack();
    }
    
    /**
     * Punto de entrada
     */
    public static void main(String args[]) {
        new Buscador().show();
    }

    public void actionPerformed(java.awt.event.ActionEvent e) {

        // La profundidad es 2 si no hay nada vlido
        int profundidad;
        try {
            profundidad=Integer.parseInt( txtProfundidad.getText());
        }
        catch (Exception exce) {
            profundidad=2;
            txtProfundidad.setText("2");
        }
        
        
        if (e.getActionCommand().equals("inet")) {
            lineas.setText("");
            lineas.append("Iniciando busqueda en internet\n");
            lineas.append("-----------------------------------------------------\n");
            lineas.append("keywords: "+txtKeyword.getText()+"\n");
            lineas.append("not-keywords: "+txtNotKeyword.getText()+"\n");
            lineas.append("-----------------------------------------------------\n");

            // Efectua la busqueda
            spider.init();
            spider.setLogger( lineas );
            spider.fillCache( txtUrl.getText() , profundidad );
            spider.search(txtKeyword.getText(), txtNotKeyword.getText());

            // Activa el boton de buscar en cache: ahora ya se puede
            buscarCache.setEnabled(true);
        }
        else { 
            lineas.setText("");
            lineas.append("Iniciando busqueda en cache\n");
            lineas.append("-----------------------------------------------------\n");
            lineas.append("keywords: "+txtKeyword.getText()+"\n");
            lineas.append("not-keywords: "+txtNotKeyword.getText()+"\n");
            lineas.append("-----------------------------------------------------\n");
            
            // Efectua la busqueda
            spider.setLogger( lineas );
            spider.search(txtKeyword.getText(), txtNotKeyword.getText());
        }
    }
}