/*-----------------------------------------------------------------------------------------  
  -- Programa:    Proxy PocketPC v1.0
  -- Autor:       Jess Ferrero Leonardo
  -- Fecha:       7-8-2002
  -- Fichero:     Spider.java
  -----------------------------------------------------------------------------------------*/

package mwa.agent;

import java.net.*;

/**
 *  Utilidades para red
 */
public class NetUtil {
    
    /** Obtiene una URI absoluta resolviendo "relativeURL" contra "baseURL" */
    public static String toAbsoluteURL(String baseURL, String relativeURL) {
        String dev=null;
        String temporalURL=null;
        boolean err=false;
        URI uribase=null,urirel=null,uriabs=null;
        
        // Crea URI base
        try {
            uribase=new URI( baseURL );
            
            if (uribase.getPath().equals("")) {
                uribase=new URI( uribase.getScheme(), 
                                  uribase.getUserInfo(), 
                                  uribase.getHost(),
                                  uribase.getPort(), 
                                  "/",
                                  uribase.getQuery(),
                                  uribase.getFragment());
            }
        }
        catch (Exception e) { 
            System.err.println("toAbsoluteURL("+baseURL+","+relativeURL+")");
            System.err.println("-> Error al procesar baseURL"); 
            System.err.println("-> Excepcion:"+e.toString());
            return baseURL;
        }

        // Crea URI relativa
        try {
            // Remplaza los " " por "+" porque es lo correcto y evita un error muy comn
            temporalURL=relativeURL.replaceAll(" ","+");
            urirel=new URI( temporalURL );
        }
        catch (Exception e) {
            System.err.println("toAbsoluteURL("+baseURL+","+relativeURL+")");
            System.err.println("-> Error al procesar relativeURL"); 
            System.err.println("-> Excepcion:"+e.toString());
            return relativeURL;
        }
        
        // Resuelve la URI relativa contra la base
        try {
            if (urirel.isAbsolute())
                dev=urirel.toString();
            else  {
                // Resuelve contra el string ya que puede haberse modificado
                // El objeto urirel sirve para verificar, nada mas
                uriabs=uribase.resolve(temporalURL);
                dev=uriabs.toString();
            }
        }
        catch (Exception uriex) {
            System.err.println("toAbsoluteURL("+baseURL+","+relativeURL+")");
            System.err.println("-> Error al resolver relativeURL contra baseURL"); 
            System.err.println("-> base resuelta: "+uribase.toString());
            System.err.println("-> rel. resuelta: "+urirel.toString());
            System.err.println("-> Excepcion:"+uriex.toString());
            return relativeURL;
        }
        // Si aparece un % en la URL, lo codifica como UTF-8 a %25
        // Ej: "http://www.google.com/espa%C3%B1ol/" + "imagen.gif"
        // =>  "http://www.google.com/espa%25C3%25B1ol/imagen.gif"
        //
        // Haciendo el siguiente replace
        // => "http://www.google.com/espa%C3%B1ol/imagen.gif"
        // Queda como debera estar
        dev=dev.replaceAll("%25","%");

        return dev;
    }
}
