/*-----------------------------------------------------------------------------------------  
  -- Programa:    Agente buscador en web usando stemming
  -- Autor:       Jess Ferrero Leonardo
  -- Fecha:       7-8-2002
  -- Fichero:     Spider.java
  -- Descripcin: Implementacin del buscador
  -----------------------------------------------------------------------------------------*/

package mwa.agent;

import javax.swing.*;
import java.io.*;
import java.util.*;

/**
 * Responsable de la bsqueda
 */
public class Spider {

    // Tamao mximo del almacen
    static final int MAX_STACK=500;
    static final int MAX_KEYWORDS=10;
    static final int MAX_HITS=10;
    int maxProfundidad;
    
    // Array para almacenar las URL
    private String lista[];
    private int tope;

    // Array para almacenar la relevancia de la url
    int fitness[];
    
    // Array de palabras clave
    String keywords[];
    String not_keywords[];
    int num_key=0;
    int num_not_key=0;

    // JTextArea para volcar los mensajes (puede ser null)
    JTextArea lineas;
    
    // TreeMap para evitar duplicados
    TreeMap urlProcesadas;
    
    // TreeMap para almacenar la stoplist (lista de palabras "prescindibles")
    TreeMap stoplist;
    
    /** Inicializa contadores, estructuras, ... */
    public void init() {
        // Inicializa el array de URLs
        lista = new String[MAX_STACK];
        fitness = new int[MAX_STACK];
        tope=0;

        // Inicializa el array de keywords
        keywords = new String[MAX_KEYWORDS];
        not_keywords = new String[MAX_KEYWORDS];
        num_key=0;
        num_not_key=0;
        
        urlProcesadas=new TreeMap();
        stoplist=new TreeMap();
        
        try {
            FileReader fr=new FileReader("stoplist.txt");
            BufferedReader br=new BufferedReader(fr);
            
            String cad=br.readLine();
            while (cad!=null) {
                if (!cad.equals("")) stoplist.put(cad.toLowerCase(),"ok");
                cad=br.readLine();
            }
            
            br.close();
            fr.close();
        }
        catch (Exception e) {
        }
    }

    /** Establece el JTextArea para volcar mensajes */
    public void setLogger(JTextArea lineas) {
        this.lineas=lineas;
    }

    /** Busqueda en profundidad de urls a partir de la direccion "url" */
    public void fillCache(String url, int profundidad) {
        // Aade la URL de partida
        lista[tope++] = url;
        urlProcesadas.put(url,"ok");
        System.err.println("Inicio URL["+(tope-1)+"]=" + url);
        
        maxProfundidad=profundidad;
        log("Leyendo documentos...");
        procesaURL(tope-1,profundidad);
        log("OK");
    }

    /** Aade a la lista todas las direcciones que encuentre en lista[pos] */
    private void procesaURL(int pos, int profundidad) {

        // Si ha alcanzado el mximo de URL permitidas para
        if (tope>=MAX_STACK) {
            System.err.println("Maximo de URL alcanzado");
            return;
        }

        String urlBase=lista[pos];
        URLdata ud=null;
        HTMLdata hd=null;

        // Slo procesa documentos que acaben en "/", "html" o "htm", el resto
        // podran ser imagenes o algo asi
        boolean esHTML = (urlBase.endsWith("/") || urlBase.toLowerCase().endsWith("html") || urlBase.toLowerCase().endsWith("htm"));

        // Si es texto
        if (esHTML) {

            // Contenedor de los datos
            ud = new URLdata();

            // Obtiene el texto del URL
            String s = ud.fetch(urlBase);
            
            // Contenedor HTML
            hd = new HTMLdata(s);
            
            // Graba el fichero en la cache
            hd.Save( getFileName(pos)+".htm" );
            hd.SaveAsText( getFileName(pos)+".txt" );
            System.err.print(tabula(profundidad)+profundidad+" "+urlBase);

            // Si profundidad=0 ya no tiene que buscar las URL de este nivel
            if (profundidad<=0) {
                System.err.println(" (fondo)");
            }
            else {
                // Obtiene una lista con todas las URL que contiene
                hd.fetchURLs();
                System.err.println(" ("+hd.NumURLs+" enlaces)");
                
                // Recorre una por una
                for (int j=0; j<hd.NumURLs; j++) {
                    // Como puede ser relativa, la resuelve
                    String urlAbsoluta= NetUtil.toAbsoluteURL( urlBase , hd.URLs[j] );

                    // Aade a la lista, pero slo si no fue procesada antes
                    if (urlProcesadas.get(urlAbsoluta)==null) {
                        lista[tope++] = urlAbsoluta;
                        urlProcesadas.put(urlAbsoluta,"ok");
                        
                        // La procesa reduciendo la profundidad
                        procesaURL(j, profundidad-1);
                    }
                    else {
                        System.err.println(tabula(profundidad)+profundidad+" "+urlAbsoluta+" ya procesada");
                    }
                }
            }
        }
    }

    private String tabula(int profundidad) {
        return new String("                  ").substring(0,maxProfundidad-profundidad);
    }
    
    /** Devuelve el nombre del fichero local en cache */
    private String getFileName(int pos) {
        return ("." + File.separator + "cache" + File.separator + "file_" + pos);
    }

    public void search(String keyWords , String notKeyWords) {
        log("Buscando...");

        // Llena el array de keywords y de notkeywords, aplicandoles el algoritmo de
        // stemming para que sean equivalentes al texto con stemming
        StringTokenizer sk;
        
        num_key=0;
        sk = new StringTokenizer(keyWords);
        
        while (sk.hasMoreTokens() && num_key<MAX_KEYWORDS) {
            keywords[num_key++] = stemming(new String(sk.nextToken())).trim();
        }

        num_not_key=0;
        sk = new StringTokenizer(notKeyWords);
        while (sk.hasMoreTokens() && num_not_key<MAX_KEYWORDS) {
            not_keywords[num_not_key++] = stemming(new String(sk.nextToken())).trim();
        }
        sk=null;

        int max_fitness=0;
        
        // Recorre todos los documentos de la lista, valorandolos por su relevancia
        for (int i=0; i<tope; i++) {
            
            // Inicialmente la peor relevancia posible
            fitness[i] = -9999;

            // Si puede, carga un String con el fichero ASCII
            // Si no puede, saltar al siguiente
            HTMLdata hdata = new HTMLdata();
            String data = hdata.Load(getFileName(i)+".txt");
            if (data==null) {
                log("Relevancia("+i+")=(no data)");
                continue;
            }
            if (data.compareTo("no document") == 0) {
                log("Relevancia("+i+")=(no data)");
                continue;
            }

            // Calcula la relevancia
            try {
                fitness[i] = relevance(data, keywords, not_keywords);
                if (fitness[i]>max_fitness) {
                    max_fitness=fitness[i];
                }
                log("Relevancia("+i+")="+fitness[i]);
            }
            catch (Exception e) {
                log("Excepcion al calcular relevancia("+i+") "+e.toString());
                e.printStackTrace();
            }
        }
        
        // Encuentra MAX_HITS los mejores documentos
        int used=0;
        int used_flag[] = new int[MAX_HITS];
        int best_fitness = 1;
        log("Resultado de la bsqueda");
        log("Keywords:");
        for (int i=0; i<num_key; i++) {
            log(keywords[i]);
        }
        
        int cuantos=10;
        for (int i=max_fitness; i>0; i--) {

            for (int j=0; j<=tope; j++) {
                if (fitness[j]==i) {
                   String input_file = getFileName(j)+".txt";
                   HTMLdata hdata = new HTMLdata();
                   String data = hdata.Load(input_file);
                   log("-------------------------------------------------");
                   log("URL["+j+"]="+lista[j]);
                   log("Relevancia="+fitness[j]);
                   log("-------------------------------------------------");
                   log(stemming(removeStopWords(data)));
                   log("-------------------------------------------------");
                   cuantos--;
                   if (cuantos==0) break;
                }
            }
            if (cuantos==0) break;
        }
        log("Fin bsqueda");
    }

    // Calculate the relevance of a file's data:
    private int relevance(String data, String keywords[], String not_keywords[]) {
        int start;
        int len = data.length();
        int count=0;

        data=removeStopWords(data);
        data=stemming(data);
        
        for (int i=0; i<num_key; i++) {
           start = 0;
           while (start<(len-2)) {
              start = data.indexOf(keywords[i],start);
              if (start==-1)  break;
              start++;
              count++;
           }
        }
        for (int i=0; i<num_not_key; i++) {
           start = 0;
           while (start<(len-2)) {
              start = data.indexOf(not_keywords[i], start);
              if (start==-1)  break;
              start++;
              count--;
           }
        }
        return count;
    }
 
    /** Elimina las palabras sin significado (stop words) del texto */
    private String removeStopWords(String data) {
        // El StringBuffer es ms eficiente
        StringBuffer res=new StringBuffer(data.length());
        StringTokenizer st=new StringTokenizer(data);
        String palabra;
        
        // Obtiene token a token, as puede comparar palabras y de paso
        // Eliminar espacios, \n, etc.
        while (st.hasMoreTokens()) {
            palabra=st.nextToken().toLowerCase();
            
            // Si es una stopword la ignora, si no, la aade a la salida
            // eliminando espacios de paso
            if (stoplist.get(palabra)==null) {
                res.append(palabra);
                res.append(" ");
            }
        }
        // Devuelve el String
        return res.toString();
    }

    /** Obtencin de la version "stemming" de una cadena de palabras.
     *  Se sigue el algoritmo de Porter:
     *  - 1) Eliminar plurales, -ed, -ing 
     *  - 2) Sustituir 'y' al final de palabra por 'i' si hay alguna otra vocal
     *  - 3) Mapear sufijos dobles a simples ... -isation 
     *  - 4) Terminaciones -ic, -full, -ness 
     *  - 5) Eliminar -ant, -ence 
     *  - 6) Eliminar -e si la palabra tiene > 2 letras */
    private String stemming(String data) {
        // El StringBuffer es ms eficiente
        StringBuffer res=new StringBuffer(data.length());
        StringTokenizer st=new StringTokenizer(data);
        String palabra;
        
        // Obtiene token a token, as puede comparar palabras y de paso
        // Eliminar espacios, \n, etc.
        while (st.hasMoreTokens()) {
            palabra=st.nextToken().toLowerCase();
            
            // 1) Eliminar -es -ed
            if (palabra.endsWith("es") || palabra.endsWith("ed")) {
                palabra=palabra.substring(0,palabra.length()-2);
            } 
            // Eliminar -ing
            else if (palabra.endsWith("ing")) {
                palabra=palabra.substring(0,palabra.length()-3);
            } 
            // 2) Sustituir 'y' al final de palabra por 'i' si hay alguna otra vocal
            else if (palabra.endsWith("y")) {
                
                if (palabra.indexOf("a")!=-1 || palabra.indexOf("e")!=-1 ||
                    palabra.indexOf("i")!=-1 || palabra.indexOf("o")!=-1 ||
                    palabra.indexOf("u")!=-1) {
                        palabra=palabra.substring(0,palabra.length()-1);
                }
            }
            // - 3) Mapear sufijos dobles a simples ... -isation 
            else if(palabra.endsWith("isation")) {
                palabra=palabra.substring(0,palabra.length()-7);
            }
            // - 4) Terminaciones -ic, -full, -ness 
            else if(palabra.endsWith("ic")) {
                palabra=palabra.substring(0,palabra.length()-2);
            }
            else if(palabra.endsWith("full") || palabra.endsWith("ness")) {
                palabra=palabra.substring(0,palabra.length()-4);
            }
            // - 5) Eliminar -ant, -ence 
            else if(palabra.endsWith("ence")) {
                palabra=palabra.substring(0,palabra.length()-4);
            }
            else if(palabra.endsWith("ant")) {
                palabra=palabra.substring(0,palabra.length()-3);
            }
            // - 6) Eliminar -e si la palabra tiene > 2 letras */
            else if(palabra.endsWith("e")) {
                if (palabra.length()>2) {
                    palabra=palabra.substring(0,palabra.length()-1);
                }
            }
            res.append(palabra);
            res.append(" ");
        }
        // Devuelve el String
        return res.toString();
    }

    private void log(String cadena) {
        lineas.append(cadena+"\n");
        lineas.paintImmediately(0, 0, lineas.getWidth(), lineas.getHeight() );
   }
}